/*
 *  Vazaar
 *  file: categories.c
 *
 *
 *  Copyright (C) 2005 Tomás Vírseda <kaskaras@yahoo.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <libxml/xmlreader.h>
#include <glade/glade.h>
#include <gnet.h>
#include <string.h>

#include "categories.h"
#include "callbacks.h"
#include "support.h"
#include "vazaar.h"
#include "debug.h"
#include "items.h"

/*
 * Solucionar esto: estas variables no deberian ser globales 
 */
extern xmlDocPtr	 cats_doc;
xmlNodePtr      	 node_found;
extern xmlDocPtr	 cats_doc;


#define DRUID_CAT_DIALOG "winCat"

/*
 * Druid Cat Signals 
 */
#define DRUIDCAT_OK_SIGNAL						"on_druidcat_ok"
#define DRUIDCAT_CANCEL_SIGNAL					"on_druidcat_cancel"
#define DRUIDCAT_BACK_SIGNAL					"on_druidcat_back"
#define DRUIDCAT_NAME_CHANGED_SIGNAL			"on_druidcat_catname_changed"
#define DRUIDCAT_TREECAT_ROW_CHANGED_SIGNAL		"on_druidCat_treecat_row_changed"
#define DRUIDCAT_CHKZLC_TOGGLED_SIGNAL			"on_druidCat_chkzlc_toggled"
#define DRUIDCAT_DELETE_EVENT_SIGNAL			"on_winCat_delete_event"


/**
 * vazaar_gui_foreach_selected_cats:
 * @model:	widget que se pasa como parámetro con la señal
 * @path:	path del treeview que se pasa como párametro con la señal
 * @iter:	iter que se pasa como párametro con la señal
 * @data:	lista de categorías seleccionadas
 *
 * Genera la lista de las categorías marcadas por el usuario.
 *
 * Los tres primeros parámetros los provee la señal
 *
 * return values: Always false. do not stop walking the store, call us with 
 * next row */
gboolean
vazaar_gui_foreach_selected_cats	(	GtkTreeModel * model,
										GtkTreePath * path,
										GtkTreeIter * iter, 
										GSList ** data)
{
    VazaarCat      *cat = NULL;

    cat = vazaar_gui_cat_get(model, *iter);
    if (cat->bool == TRUE) {
		*data = g_slist_append(*data, cat->name);
		if (DEBUG)
			g_message("%s => %d, %s", __FUNCTION__, cat->bool, cat->name);
	}
    g_free(cat);
    return FALSE;
}

/**
 * vazaar_gui_foreach_hide_category:
 * @model:	widget que se pasa como parámetro con la señal
 * @path:	path del treeview que se pasa como párametro con la señal
 * @iter:	iter que se pasa como párametro con la señal
 * @data:	lista de categorías seleccionadas
 * Los tres primeros parámetros los provee la señal
 *
 * Hide category to edit. Return TRUE and exit if is found.
 *
 * return values: Always false. do not stop walking the store, call us with 
 * next row */
gboolean
vazaar_gui_foreach_hide_category	(	GtkTreeModel * model,
				  						GtkTreePath * path,
				  						GtkTreeIter * iter, 
										gchar *category)
{
    VazaarCat      *cat = NULL;

    cat = vazaar_gui_cat_get(model, *iter);
    if (vazaar_cmd_strings_compare (cat->name, category) == 0) {
		gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
		g_free(cat);
		return TRUE;
	}
    g_free(cat);
    return FALSE;		/* do not stop walking the store, call us with next row */
}

/**
 * vazaar_gui_foreach_select_parent_category:
 * @model:	widget que se pasa como parámetro con la señal
 * @path:	path del treeview que se pasa como párametro con la señal
 * @iter:	iter que se pasa como párametro con la señal
 * @data:	pointer a tres punteros (documentar)
 * Los tres primeros parámetros los provee la señal
 *
 * Select parent category. Return TRUE and exit if is found.
 *
 * return values: Always false. do not stop walking the store, call us with 
 * next row */
gboolean
vazaar_gui_foreach_select_parent_category	(	GtkTreeModel * model,
				  								GtkTreePath * path,
				  								GtkTreeIter * iter, 
												gpointer data[1])
{
    VazaarCat		*cat = NULL;
	GtkTreeView		*treeview;
	GtkWidget		*rbParent;
	gchar			*parent;
	gint			 depth;
	
	treeview	= data[0];
	parent		= data[1];
	rbParent	= data[2];
	
    cat = vazaar_gui_cat_get(model, *iter);
    if (vazaar_cmd_strings_compare (cat->name, parent) == 0) {
		GtkTreeSelection	*selection;
		selection = gtk_tree_view_get_selection(treeview);
		gtk_tree_selection_select_iter (selection, iter);
		depth = gtk_tree_path_get_depth (path);
		if ( depth > 0) {
			gtk_widget_set_sensitive (GTK_WIDGET (treeview), TRUE);
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rbParent), FALSE);
		}
		g_free(cat);
		return TRUE;
	}
    g_free(cat);
    return FALSE;	/* do not stop walking the store, call us with next row */
}

/**
 * vazaar_gui_foreach_cat_in_item:
 * @model:	widget que se pasa como parámetro con la señal
 * @path:	path del treeview que se pasa como párametro con la señal
 * @iter:	iter que se pasa como párametro con la señal
 * @data:	nombre de la categoria
 * Los tres primeros parámetros los provee la señal
 *
 * Select parent category. Return TRUE and exit if is found.
 *
 * return values: Always false. do not stop walking the store, call us with 
 * next row */
gboolean
vazaar_gui_foreach_cat_in_item	(	GtkTreeModel * model,
				  					GtkTreePath * path,
				  					GtkTreeIter * iter, 
									gchar *category)
{
    VazaarCat      *cat = NULL;

    cat = vazaar_gui_cat_get(model, *iter);
	if (vazaar_cmd_strings_compare (cat->name, category) == 0) {
		gtk_tree_store_set (GTK_TREE_STORE (model), iter, CAT_COL_CHK, TRUE, -1);
	}

    g_free(cat);
	
	/* do not stop walking the store, call us with next row */
    return FALSE;		
}

/* 
 * XML functions
 */

/**
 * vazaar_xml_cat_add:
 * @category:	categoria que se quiere añadir
 *
 * Añade una categoria en formato XML
 *
 * return values: no return value
 */
void
vazaar_xml_cat_add(VazaarCat * category)
{
    xmlNodePtr      root,
                    child,
                    parent;

    root = xmlDocGetRootElement(cats_doc);
    node_found = NULL;
    vazaar_xml_cat_find(root->xmlChildrenNode, category->root);
    parent = node_found;

    if (!parent)
	parent = root;

    child = xmlNewChild(parent, NULL, "category", NULL);
    xmlSetProp(child, "icon", category->icon);
    xmlSetProp(child, "name", category->name);
    xmlSetProp(child, "desc", category->desc);
	xmlSetProp(child, "syst", category->syst);
    vazaar_xml_cats_savefile();
    if (DEBUG)
		g_message("vazaar_xml_cat_add: add new category: %s", category->name);

}

/**
 * vazaar_xml_cat_del:
 * @name:	nombre de la categoría que se quiere eliminar
 *
 * Borra la categoría del archivo xml
 * ¡¡¡ Debería cambiarse por el ID de la categoría !!!
 *
 * return values: TRUE if successful, FALSE if not.
 */
gboolean
vazaar_xml_cat_del(const gchar * name, gboolean force)
{
    gboolean         erase = FALSE;
    xmlNodePtr       root,
                     child;
	gint			 cmp = -1;
	VazaarCat		*cat;
	
    node_found = NULL;

    root = xmlDocGetRootElement(cats_doc);
    vazaar_xml_cat_find(root->xmlChildrenNode, g_strdup(name));
    child = node_found;

	/* This is only used to fix bugs */
	if ((force) && (child)) {
			xmlUnlinkNode(child);
			vazaar_xml_cats_savefile();
			return TRUE;
	}
	
    if (child) {
		if (child->children) {
	    	erase = FALSE;
		} else {
			cat = vazaar_xml_cat_get (child);
			cmp = vazaar_cmd_strings_compare (cat->syst, "false");
			if (cmp == 0) {
	    		xmlUnlinkNode(child);
	    		vazaar_xml_cats_savefile();
	    		erase = TRUE;
			}
			else {
				erase = FALSE;
			}
		}
   }

    return erase;
}

/**
 * vazaar_xml_cat_get:
 * @node:	nodo xml de la categoría
 *
 * Extrae la información del nodo xml
 *
 * return values: VazaarCat struct with the info.
 */
VazaarCat *
vazaar_xml_cat_get(xmlNodePtr node)
{
    VazaarCat	*cat;
	VazaarCat	*pcat;

    cat = g_new0(VazaarCat, 1);
	pcat = g_new0 (VazaarCat, 1);
	pcat = vazaar_xml_cat_get_parent (node->parent);
	
    cat->icon = g_strdup(xmlGetProp(node, "icon"));
    cat->name = g_strdup(xmlGetProp(node, "name"));
    cat->desc = g_strdup(xmlGetProp(node, "desc"));	
	cat->root = g_strdup_printf ("%s", pcat->name);
	cat->syst = g_strdup(xmlGetProp(node, "syst"));
	g_free (pcat);
    return cat;
}

VazaarCat      *
vazaar_xml_cat_get_parent(xmlNodePtr node)
{
    VazaarCat	*cat;

    cat = g_new0(VazaarCat, 1);
	
    cat->icon = g_strdup(xmlGetProp(node, "icon"));
    cat->name = g_strdup(xmlGetProp(node, "name"));
    cat->desc = g_strdup(xmlGetProp(node, "desc"));
	cat->syst = g_strdup(xmlGetProp(node, "syst"));
    return cat;
}

void
vazaar_xml_cat_find(xmlNodePtr a_node, gchar * name)
{
    xmlNodePtr      cur_node = NULL;
    VazaarCat      *cat;
    gint            cmp = -1;

    for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
	if (cur_node->type == XML_ELEMENT_NODE) {
	    cat = vazaar_xml_cat_get(cur_node);
	    cmp = vazaar_cmd_strings_compare(name, cat->name);
	    if (cmp == 0)
		node_found = cur_node;
	}

	vazaar_xml_cat_find(cur_node->children, name);
    }
}

int
vazaar_xml_cats_savefile()
{
    gchar          *doc_path;
    int             res;

    doc_path = vazaar_xml_catdoc_get();
    res = xmlSaveFile(doc_path, cats_doc);
    g_free(doc_path);
    return res;
}


void
vazaar_xml_parse_categories	(	GtkTreeStore * model, 
								GtkTreeIter * parent,
								xmlNodePtr node, 
								gboolean all_cats)
{
    VazaarCat      *cat, *pcat;	
    GtkTreeIter     iter = { 0 };
    GdkPixbuf      *icon = NULL;
	GtkTreePath		*iter_path,
					*parent_path;
	gchar *tmp_name;
	
    while (node) {
		cat = vazaar_xml_cat_get(node);
		//pcat = vazaar_xml_cat_get_parent (node->parent);
		//g_message ("Parent: %s, Children: %s", pcat->name, cat->name);
		if (all_cats) {	
			icon = create_pixbuf(cat->icon);
			gtk_tree_store_append(model, &iter, parent);
			gtk_tree_store_set	(	GTK_TREE_STORE(model), &iter, 
									CAT_COL_IMG, icon,
					   				CAT_COL_CHK, FALSE, 
									CAT_COL_NOM, cat->name,
					   				CAT_COL_DES, cat->desc, 
									CAT_COL_SYS, cat->syst, 
									-1);
			if (DEBUG)
				g_message("%s => Parent: %s", __FUNCTION__, cat->name);
		}
		else {
			gint cmp;
			cmp = vazaar_cmd_strings_compare (cat->syst, "false");
			if (cmp == 0) {
				icon = create_pixbuf(cat->icon);
				gtk_tree_store_append(model, &iter, parent);
				gtk_tree_store_set	(	GTK_TREE_STORE(model), &iter, 
										CAT_COL_IMG, icon,
						   				CAT_COL_CHK, FALSE, 
										CAT_COL_NOM, cat->name,
						   				CAT_COL_DES, cat->desc, 
										-1);
				if (DEBUG)
					g_message ("%s => Parent: %s", __FUNCTION__, cat->name);
			}
		}		
		
		for (node = node->xmlChildrenNode; node; node = node->next) {
			//g_message ("XML_TEXT_NODE: %d / Current: %d", XML_TEXT_NODE, node->type);
			if (node->type != XML_TEXT_NODE) {
				vazaar_xml_parse_categories(model, &iter, node, all_cats);
			}
		}	
    }
}

void
vazaar_xml_druidcat_parse_categories	(	GtkTreeStore	*model, 
											GtkTreeIter		*parent,
											xmlNodePtr		 node)
{
    VazaarCat      *cat;
    GtkTreeIter     iter = { 0 };
    GdkPixbuf      *icon = NULL;
	gint			cmp = -1;

    while (node) {
		cat = vazaar_xml_cat_get(node);
		cmp = vazaar_cmd_strings_compare (cat->syst, "false");
		if (cmp == 0) {
			icon = create_pixbuf(cat->icon);
			gtk_tree_store_append(model, &iter, parent);
			gtk_tree_store_set	(	GTK_TREE_STORE(model), &iter, 
									CAT_COL_IMG, icon,
					   				CAT_COL_CHK, FALSE, 
									CAT_COL_NOM, cat->name,
					   				CAT_COL_DES, cat->desc, 
									CAT_COL_SYS, cat->syst,
									-1);
			if (DEBUG) {
				g_message("%s => Parent: %s", __FUNCTION__, cat->name);
			}
			
			for (node = node->xmlChildrenNode; node; node = node->next) {
				if (node->type != XML_TEXT_NODE) {
					vazaar_xml_druidcat_parse_categories(model, &iter, node);
				}
			}
		}
    }
}

GtkTreeModel   *
vazaar_gui_catmodel_create_and_fill()
{
    GtkTreeSortable *sortable;
    GtkTreeStore   *model = NULL;
    xmlNode        *root,
                   *child;
    VazaarCat      *cat;


    cat = (VazaarCat *) NULL;

    model = gtk_tree_store_new(CAT_NUM_COLS,	/* Number of columns */
			       GDK_TYPE_PIXBUF,	/* categories Icon */
			       G_TYPE_BOOLEAN,	/* choosed? */
			       G_TYPE_STRING,	/* Name */
					G_TYPE_STRING,	/* System Name */
			       G_TYPE_STRING,	/* Descripcion */
					G_TYPE_STRING);	/* Descripcion */
    sortable = GTK_TREE_SORTABLE(model);
    gtk_tree_sortable_set_sort_column_id(sortable, CAT_COL_NOM,
					 GTK_SORT_ASCENDING);

    root = xmlDocGetRootElement(cats_doc);
    for (child = root->children; child; child = child->next)
		vazaar_xml_parse_categories(model, NULL, child, TRUE);

    g_assert(model);
    return GTK_TREE_MODEL(model);
}

GtkTreeModel   *
vazaar_gui_druidcatmodel_create_and_fill()
{
    GtkTreeSortable *sortable;
    GtkTreeStore   *model = NULL;
    xmlNode        *root,
                   *child;
    VazaarCat      *cat;


    cat = (VazaarCat *) NULL;

    model = gtk_tree_store_new(CAT_NUM_COLS,	/* Number of columns */
			       GDK_TYPE_PIXBUF,	/* categories Icon */
			       G_TYPE_BOOLEAN,	/* choosed? */
			       G_TYPE_STRING,	/* Name */
			       G_TYPE_STRING,	/* System Name */
					G_TYPE_STRING,  /* Descripcion */
					G_TYPE_STRING);	/* system */
    sortable = GTK_TREE_SORTABLE(model);
    gtk_tree_sortable_set_sort_column_id(sortable, CAT_COL_NOM,
					 GTK_SORT_ASCENDING);

    root = xmlDocGetRootElement(cats_doc);
    for (child = root->children; child; child = child->next)
		vazaar_xml_parse_categories(model, NULL, child, FALSE);

    g_assert(model);
    return GTK_TREE_MODEL(model);
}


void
vazaar_gui_catmodel_createview_sel(GtkTreeModel * model,
			       GtkTreeView * treeview)
{
    GtkTreeViewColumn *column_icon,
                   *column_text,
                   *column_checked,
                   *column_desc;
    GtkCellRenderer *cell_icon,
                   *cell_text,
                   *cell_checked,
                   *cell_desc;

    g_assert(treeview);


    /*
     * CATEGORY ICON COLUMN 
     */
    column_icon = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_icon, FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_icon);
    cell_icon = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_end(column_icon, cell_icon, TRUE);
    gtk_tree_view_column_add_attribute(column_icon, cell_icon, "pixbuf",
				       CAT_COL_IMG);

    /*
     * CATEGORY CHK COLUMN 
     */
    column_checked = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_checked, FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_checked);
    cell_checked = gtk_cell_renderer_toggle_new();
    gtk_tree_view_column_set_visible(column_checked, FALSE);

    /*
     * CATEGORY TEXT COLUMN 
     */
    column_text = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_text, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_text);
    cell_text = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_text, cell_text, TRUE);
    gtk_tree_view_column_add_attribute(column_text, cell_text, "text",
				       CAT_COL_NOM);

    /*
     * CATEGORY DESC COLUMN 
     */
    column_desc = gtk_tree_view_column_new();
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_desc);
    cell_desc = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_desc, cell_desc, TRUE);
    gtk_tree_view_column_add_attribute(column_desc, cell_desc, "text",
				       CAT_COL_NOM);
    gtk_tree_view_column_add_attribute(column_desc, cell_desc, "visible",
				       CAT_COL_DES);
    gtk_tree_view_column_set_visible(column_desc, FALSE);

    gtk_tree_view_column_pack_end(column_checked, cell_checked, TRUE);
    gtk_tree_view_column_add_attribute(column_checked, cell_checked,
				       "active", CAT_COL_CHK);
    g_signal_connect(cell_checked, "toggled",
		     G_CALLBACK(on_toggle_categorie), model);


    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview),
			    GTK_TREE_MODEL(model));
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection
				(GTK_TREE_VIEW(treeview)),
				GTK_SELECTION_BROWSE);
    gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview));
}


void
vazaar_gui_catmodel_createview(GtkTreeModel * model,
			       GtkTreeView * treeview)
{
    GtkTreeViewColumn *column_icon,
                   *column_text,
					*column_i18n,
                   *column_checked,
                   *column_desc,
					*column_syst;
    GtkCellRenderer *cell_icon,
                   *cell_text,
					*cell_i18n,
                   *cell_checked,
                   *cell_desc,
					*cell_syst;
    g_assert(treeview);


    /*
     * CATEGORY ICON COLUMN 
     */
    column_icon = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_icon, FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_icon);
    cell_icon = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_end(column_icon, cell_icon, TRUE);
    gtk_tree_view_column_add_attribute(column_icon, cell_icon, "pixbuf",
				       CAT_COL_IMG);

    /*
     * CATEGORY CHK COLUMN 
     */
    column_checked = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_checked, FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_checked);
    cell_checked = gtk_cell_renderer_toggle_new();
    gtk_tree_view_column_set_visible(column_checked, FALSE);

    /*
     * CATEGORY TEXT COLUMN 
     */
    column_text = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_text, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_text);
    cell_text = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_text, cell_text, TRUE);
    gtk_tree_view_column_add_attribute(column_text, cell_text, "text",
				       CAT_COL_NOM);


    /*
     * CATEGORY DESC COLUMN 
     */
    column_desc = gtk_tree_view_column_new();
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_desc);
    cell_desc = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_desc, cell_desc, TRUE);
    gtk_tree_view_column_add_attribute(column_desc, cell_desc, "text", CAT_COL_DES);
    gtk_tree_view_column_add_attribute(column_desc, cell_desc, "visible", CAT_COL_DES);
    gtk_tree_view_column_set_visible(column_desc, FALSE);

    gtk_tree_view_column_pack_end(column_checked, cell_checked, TRUE);
    gtk_tree_view_column_add_attribute(column_checked, cell_checked,
				       "active", CAT_COL_CHK);
    g_signal_connect(cell_checked, "toggled",
		     G_CALLBACK(on_toggle_categorie), model);

    /*
     * CATEGORY SYST COLUMN 
     */
    column_syst = gtk_tree_view_column_new();
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_syst);
    cell_syst = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_syst, cell_syst, TRUE);
    gtk_tree_view_column_add_attribute(column_syst, cell_syst, "text", CAT_COL_SYS);
	gtk_tree_view_column_set_visible(column_syst, FALSE);

    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview),
			    GTK_TREE_MODEL(model));
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection
				(GTK_TREE_VIEW(treeview)),
				GTK_SELECTION_BROWSE);
    gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview));
}

void
vazaar_gui_catmodel_refresh(GtkTreeView * treecat)
{
    GtkTreeModel   *model;
    xmlNode        *root, *child;

    model = gtk_tree_view_get_model(treecat);
    gtk_tree_store_clear(GTK_TREE_STORE(model));

    root = xmlDocGetRootElement(cats_doc);
	for (child = root->children; child; child = child->next)
    	vazaar_xml_parse_categories(GTK_TREE_STORE(model), NULL, child, TRUE);
	// Modifify this behaviour with GConf retrieving collapsed/expanded rows
    gtk_tree_view_expand_all(treecat);
}

/* GtkIconView Icon model */
GtkListStore   *
vazaar_gui_iconmodel_create()
{
	GtkListStore   *model = NULL;
	//gchar				*icon_file = NULL;
	
    model = gtk_list_store_new (2,	/* Number of columns */
			       GDK_TYPE_PIXBUF,	/* categories Icon */
			       G_TYPE_STRING);	/* Name */


    g_assert(model);
    return GTK_LIST_STORE (model);
}

void vazaar_gui_iconmodel_fill (GtkListStore *model) {
	GtkTreeIter			 iter = { 0 };
	GdkPixbuf			*gdkicon = NULL;
	GList				*files;
	GnomeVFSFileInfo	*icon = NULL;
	
	files = vazaar_cmd_get_files_from_dir (PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/user/");
	
	while (files) {
		icon = (GnomeVFSFileInfo *)files->data;
		if (icon->type == GNOME_VFS_FILE_TYPE_REGULAR) {
			gdkicon = create_pixbuf(icon->name);
			if (gdkicon) {
				gtk_list_store_append(model, &iter);
				gtk_list_store_set	(	GTK_LIST_STORE(model), &iter, 
										0, gdkicon,
										1, icon->name,
										-1);
			}
		}
		files = files->next;
	}
}

void
vazaar_gui_iconmodel_createview(GtkTreeModel * model,
			       GtkTreeView * treeview)
{
    GtkTreeViewColumn *column_icon,
                   *column_text;
    GtkCellRenderer *cell_icon,
                   *cell_text;

    g_assert(treeview);

    /*
     * CATEGORY ICON COLUMN 
     */
    column_icon = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_icon, FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_icon);
    cell_icon = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_end(column_icon, cell_icon, TRUE);
    gtk_tree_view_column_add_attribute(column_icon, cell_icon, "pixbuf",
				       CAT_COL_IMG);

    /*
     * CATEGORY TEXT COLUMN 
     */
    column_text = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_text, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_text);
    cell_text = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_text, cell_text, TRUE);
    gtk_tree_view_column_add_attribute(column_text, cell_text, "text",
				       CAT_COL_NOM);
					   
    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview),
			    GTK_TREE_MODEL(model));
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection
				(GTK_TREE_VIEW(treeview)),
				GTK_SELECTION_BROWSE);
}

void
vazaar_gui_cat_add(Vazaar * app)
{
    VazaarDruidCat *druid;

    druid = vazaar_gui_druidcat_new(app, NULL);
}

void
vazaar_gui_cat_edit(Vazaar * app)
{
    xmlNodePtr      root,
                    found;
    GtkTreeSelection *selection;
    GtkTreeModel   *model;
    GtkTreeIter     iter;
    gchar          *name;
	gint			cmp;
    VazaarCat      *cat = NULL;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treecat));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
	gtk_tree_model_get(model, &iter, CAT_COL_NOM, &name, -1);
	root = xmlDocGetRootElement(cats_doc);
	node_found = NULL;
	vazaar_xml_cat_find(root->xmlChildrenNode, name);
	found = node_found;
	if (found) {
	    cat = vazaar_xml_cat_get(found);
		cmp = vazaar_cmd_strings_compare (cat->syst, "false");
		if (cmp == 0) {
			vazaar_gui_druidcat_new(app, cat);
	    	if (DEBUG)
				g_message("%s => %s", __FUNCTION__, cat->name);
	    }
		else
			gnome_appbar_set_status(GNOME_APPBAR (app->statusbar), _("You can't edit a system category"));
	}
	g_free(name);
    }
}

void
vazaar_gui_cat_delete(Vazaar * app)
{
    GtkTreeSelection *selection;
    GtkTreeModel   *model;
    GtkTreeIter     iter;
    GtkWidget      *dialog;
    gchar          *title,
					*details,
					*str,
                   *name;
    gint            result;
    gboolean        erase = FALSE;

    /*
     * This will only work in single or browse selection mode! 
     */	
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treecat));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, CAT_COL_NOM, &name, -1);
		str = g_strdup_printf (_("Deleting category '%s'"), name);
		gnome_appbar_set_status (GNOME_APPBAR (app->statusbar), str);	
		title = g_strdup_printf(_("Delete '%s' category?"), name);
		details = g_strdup_printf (_("All items will be moved to wastebasket"));
		result = vazaar_gui_show_dialog (app->window, GTK_STOCK_DIALOG_WARNING, title, details);
		switch (result) {
		case GTK_RESPONSE_OK:
			model = gtk_tree_view_get_model(GTK_TREE_VIEW (app->treecat));
			erase = vazaar_xml_cat_del(name, FALSE);
			if (erase) {
				/* Move category's items to Wastebasket category or delete */
				vazaar_cmd_move_cat_to_wastebasket (app, name);
				gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
				str = g_strdup_printf (_("Category '%s' deleted"), name);
				gnome_appbar_set_status (GNOME_APPBAR (app->statusbar), str);		
			} 
			else {
				gnome_appbar_set_status (GNOME_APPBAR (app->statusbar), _("Category not deleted"));	
				title = g_strdup_printf (_("Operation not allowed"));
				details = g_strdup_printf (_("You can't delete categories with subcategories or 'system' categories"));
				vazaar_gui_show_dialog (app->window, GTK_STOCK_DIALOG_ERROR, title, details);
				
			}
			break;
		case GTK_RESPONSE_CANCEL:
			gnome_appbar_set_status (GNOME_APPBAR (app->statusbar), _("Category not deleted"));	
			break;
		}
		g_free(name);
    } 
	else {
		if (DEBUG)
			g_message("no row selected.\n");
		}
}

void
vazaar_gui_cat_load(Vazaar * app)
{
    GtkTreeModel   *model;

    if (DEBUG)
	g_message("Loading and filling categories model...");
    model = vazaar_gui_catmodel_create_and_fill();
    if (model) {
	vazaar_gui_catmodel_createview(model, GTK_TREE_VIEW (app->treecat));
    } else {
	g_error(_("Categories model doesn't exists"));
    }
    g_object_unref(model);	/* destroy model automatically with view */
}

VazaarCat      *
vazaar_gui_cat_get(GtkTreeModel * model, GtkTreeIter iter)
{
    VazaarCat      *category;

    category = g_new0(VazaarCat, 1);
    gtk_tree_model_get(model, &iter,
		       // CAT_COL_IMG, icon,
		       CAT_COL_CHK, &category->bool,
		       CAT_COL_NOM, &category->name,
		       CAT_COL_DES, &category->desc, -1);

    return category;
}

GSList         *
vazaar_gui_cat_selected(GtkWidget * treeview)
{
    GtkTreeModel   *model;
    GSList         *select_cats = NULL;
    GtkTreeIter     iter = { 0 };
    gboolean        pass = FALSE;
    VazaarCat      *cat;

    g_assert(treeview);

    model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
    pass = gtk_tree_model_get_iter_first(model, &iter);
    while (pass) {
		cat = vazaar_gui_cat_get(model, iter);
		if (cat->bool == TRUE) {
			select_cats = g_slist_append(select_cats, cat->name);
		}
		g_free(cat);
		pass = gtk_tree_model_iter_next(model, &iter);
    }
    return select_cats;
}


VazaarDruidCat *
vazaar_gui_druidcat_new(Vazaar * app, VazaarCat * cat)
{
    GtkTextBuffer  *buffer;
    VazaarDruidCat *this;
    GtkTreeModel   *model;
    gchar          *str;
	GtkListStore	*icon_model;
	
	/* winMain insensitive */
	gtk_widget_set_sensitive (app->window, FALSE);
	
    this = g_new0(VazaarDruidCat, 1);

    this->xml =
	glade_xml_new(GLADE_VAZAAR_PROJECT_FILE, DRUID_CAT_DIALOG, NULL);
    this->dialog = glade_xml_get_widget(this->xml, DRUID_CAT_DIALOG);
    this->wparent = app->window;
    this->treecat = app->treecat;
    this->treeselcat = glade_xml_get_widget(this->xml, "treecat");
	this->frmCats = glade_xml_get_widget(this->xml, "vbox55");
	g_assert (this->frmCats);
    this->name = glade_xml_get_widget(this->xml, "name");
    this->desc = glade_xml_get_widget(this->xml, "desc");
    this->rbParentCat = glade_xml_get_widget(this->xml, "chkZLCat");
	this->iconview = glade_xml_get_widget(this->xml, "iconview");
//    this->iconsel = glade_xml_get_widget(this->xml, "iconsel");
	this->btnOK = glade_xml_get_widget(this->xml, "btnOK");
	this->btnCancel = glade_xml_get_widget(this->xml, "btnCancel");
	this->icon_path = NULL;
	/* Categories model */
    model = vazaar_gui_druidcatmodel_create_and_fill();
    if (model)
		vazaar_gui_catmodel_createview (model, GTK_TREE_VIEW(this->treeselcat));

    if (cat)
		this->editcat = g_strdup(cat->name);
	
    
	/* Iconview model */
	icon_model = vazaar_gui_iconmodel_create ();
	vazaar_gui_iconmodel_fill (icon_model);
	this->iconsel = gtk_icon_view_new_with_model (GTK_TREE_MODEL(icon_model));
	gtk_container_add (GTK_CONTAINER (this->iconview), this->iconsel); 
	gtk_widget_show (this->iconview);
	gtk_widget_show (this->iconsel);
	gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (this->iconsel), 0);
	
	gtk_icon_view_set_row_spacing   (GTK_ICON_VIEW (this->iconsel), 20);
	gtk_icon_view_set_column_spacing (GTK_ICON_VIEW (this->iconsel), 15);
	gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (this->iconsel), GTK_SELECTION_MULTIPLE);
	g_object_unref (icon_model);
	
	this->new_cat = g_new0(VazaarCat, 1);
	this->new_cat->syst = g_strdup_printf ("%s", "false");
	gtk_window_set_title(GTK_WINDOW(this->dialog), "");
	gtk_widget_set_sensitive (this->btnOK, FALSE);
	
	/* Conect signals */
	vazaar_gui_druidcat_connect_signals(this);
	
    if (this->editcat) {
		//gchar 		*tmp = NULL;
		gpointer	 data[2];
		data[0] = this->treeselcat;
		data[1] = cat->root;
		data[2] = this->rbParentCat;
		
		gtk_tree_model_foreach	(	model, 
									(GtkTreeModelForeachFunc)vazaar_gui_foreach_hide_category,
			   						this->editcat);
		gtk_tree_model_foreach	(	model, 
									(GtkTreeModelForeachFunc)vazaar_gui_foreach_select_parent_category,
			   						data);
		
		str = g_strdup_printf(_("Editing category '%s'"), this->editcat);
		gtk_window_set_title(GTK_WINDOW(this->dialog), str);
		gtk_entry_set_text(GTK_ENTRY (this->name), cat->name);
		buffer = gtk_text_buffer_new(NULL);
		gtk_text_buffer_set_text(buffer, cat->desc, -1);
		gtk_text_view_set_buffer(GTK_TEXT_VIEW (this->desc), buffer);
		this->new_cat->root = g_strdup_printf ("%s", cat->root);
		this->new_cat->icon = g_strdup_printf ("%s", cat->icon);
		vazaar_gui_iconmodel_set_icon (this);
									
		//gtk_icon_view_item_activated (GTK_ICON_VIEW (this->iconsel), this->icon_path);
		
		//g_free (tmp);
    } 
	else {
		gtk_widget_show (this->btnOK);
		gtk_window_set_title (GTK_WINDOW (this->dialog), _("Adding new category..."));
    }
	
    gtk_window_set_transient_for(GTK_WINDOW(this->dialog), GTK_WINDOW(this->wparent));
    vazaar_gui_druidcat_show(this);

    return this;
}

void
vazaar_gui_druidcat_connect_signals(VazaarDruidCat * druidcat)
{
    glade_xml_signal_connect_data	(	druidcat->xml, 
										DRUIDCAT_CANCEL_SIGNAL,
				  						GTK_SIGNAL_FUNC(vazaar_gui_druidcat_cancel), 
										druidcat);
	
	glade_xml_signal_connect_data	(	druidcat->xml, DRUIDCAT_OK_SIGNAL,
				  						GTK_SIGNAL_FUNC (vazaar_gui_druidcat_finish), 
										druidcat);
	
    glade_xml_signal_connect_data	(	druidcat->xml,
				  						DRUIDCAT_NAME_CHANGED_SIGNAL,
				  						GTK_SIGNAL_FUNC (vazaar_gui_druidcat_catname_changed),
				  						druidcat);
	
    glade_xml_signal_connect_data	(	druidcat->xml,
				  						DRUIDCAT_CHKZLC_TOGGLED_SIGNAL,
				  						GTK_SIGNAL_FUNC (vazaar_gui_druidCat_chkzlc_toggled),
				  						druidcat);
	
    glade_xml_signal_connect_data	(	druidcat->xml,
				  						DRUIDCAT_DELETE_EVENT_SIGNAL,
				  						GTK_SIGNAL_FUNC (vazaar_gui_druidCat_delete_event),
				  						druidcat);

	g_signal_connect				(	druidcat->iconsel,
										"selection-changed",
			    						G_CALLBACK (vazaar_gui_iconmodel_get_icon), druidcat);
}

gboolean
vazaar_cmd_iconmodel_traverse (	GtkTreeModel * model,
				  				GtkTreePath * path,
				  				GtkTreeIter * iter, 
								VazaarDruidCat	*druidcat) {

	gchar *icon;
									
	gtk_tree_model_get_iter (GTK_TREE_MODEL (model), iter, path);
	gtk_tree_model_get (GTK_TREE_MODEL (model), iter, 1, &icon, -1);		
	if (vazaar_cmd_strings_compare (druidcat->new_cat->icon, icon) == 0) {
		druidcat->icon_path = gtk_tree_path_copy (path);	
		return TRUE;		
	}
									
	return FALSE;	
}

void
vazaar_gui_iconmodel_set_icon (	VazaarDruidCat *	this) {
	
	GtkTreePath	*tree_path = NULL;
	
	gtk_tree_model_foreach	(	GTK_TREE_MODEL (gtk_icon_view_get_model (GTK_ICON_VIEW (this->iconsel))), 
								(GtkTreeModelForeachFunc)vazaar_cmd_iconmodel_traverse, 
								this);
	
	gtk_icon_view_select_path (GTK_ICON_VIEW (this->iconsel), this->icon_path);	
}


gchar *
vazaar_gui_iconmodel_get_icon (GtkWidget *nothing, VazaarDruidCat *this)
{
	GtkListStore	*store;
	GtkTreePath		*tree_path = NULL;
	GtkTreeIter 	 iter;
	GList			*items = NULL;
	gchar			*icon = NULL;
	
	g_assert (this);

	store = GTK_LIST_STORE (gtk_icon_view_get_model (GTK_ICON_VIEW (this->iconsel)));
	items = gtk_icon_view_get_selected_items (GTK_ICON_VIEW (this->iconsel));
	/* I only select the first item selectd (there is only one) */
	if (items) {
		tree_path = items->data;
		this->icon_path = gtk_tree_path_copy (tree_path);
		gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, tree_path);
		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 1, &icon, -1);
	}
	
	return icon;
}

void
vazaar_gui_druidCat_delete_event(GtkWindow * win,
				 VazaarDruidCat * druidcat)
{
    return;
}

void
vazaar_gui_druidcat_show(VazaarDruidCat *this)
{
    gtk_widget_show (this->dialog);
}

void
vazaar_gui_druidcat_cancel(GtkWidget * nothing, VazaarDruidCat *this)
{
    vazaar_gui_druidcat_destroy(this);
}

void
vazaar_gui_druidcat_destroy(VazaarDruidCat * this)
{
	gtk_widget_set_sensitive(this->wparent, TRUE);
    gtk_widget_destroy(this->dialog);
    g_free(this->new_cat);
    g_free(this);
}

void
vazaar_gui_druidcat_catname_changed(GtkWidget * entry,
				    VazaarDruidCat * this)
{
    g_assert(this);
    const gchar    *name = gtk_entry_get_text(GTK_ENTRY (this->name));
	
    if (g_ascii_strcasecmp(name, "") == 0) {
		gtk_widget_set_sensitive (this->btnOK, FALSE);
		gtk_window_set_title (GTK_WINDOW (this->dialog), _("Adding new category..."));
	}
	else {
		gchar *tmp = NULL;
		gtk_widget_set_sensitive (this->btnOK, TRUE);
		if (this->editcat)
			tmp = g_strdup_printf (_("Editing category: '%s'"), name);
		else
			tmp = g_strdup_printf (_("Adding new category: '%s'"), name);
		gtk_window_set_title (GTK_WINDOW (this->dialog), tmp);
		g_free (tmp);
	}
}

void
vazaar_gui_druidcat_resume(VazaarDruidCat * this)
{
    GtkTreeSelection *selection;
    GtkTreeModel   *model;
    GtkTreeIter     iter;
    GtkTextBuffer  *buffer;
    GtkTextIter     start,
                    end;
    gchar			*name,
					*icon;

    /*
     * PAGE ONE 
     */
    this->new_cat->name = g_strdup(gtk_entry_get_text(GTK_ENTRY (this->name)));
    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (this->desc));
    gtk_text_buffer_get_start_iter(buffer, &start);
    gtk_text_buffer_get_end_iter(buffer, &end);
    this->new_cat->desc =
	gtk_text_buffer_get_text(buffer, &start, &end, FALSE);

	icon = vazaar_gui_iconmodel_get_icon (NULL, this);
	if (!icon) {
		if (!this->editcat) {    	
			this->new_cat->icon = g_strdup_printf("%s", "penguin.png");
		}
	}
	else {
		this->new_cat->icon = g_strdup (icon);
	}
    g_free (icon);

    /*
     * PAGE TWO 
     */
    gboolean        parent_cat = FALSE;

	parent_cat = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(this->rbParentCat));
	if (parent_cat) {
		this->new_cat->root = g_strdup_printf("%s", "");
	} 
	else {
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(this->treeselcat));
		if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
			gtk_tree_model_get(model, &iter, CAT_COL_NOM, &name, -1);
			this->new_cat->root = g_strdup_printf("%s", name);
		}
	}
	
	/* Bug fixed: check conditions always */
	if ((!parent_cat) && (!this->new_cat->root)) {
		this->new_cat->root = g_strdup_printf("%s", "");
	}
		
    if (DEBUG)
		g_message("%s, %s, %s", this->new_cat->name, this->new_cat->desc, this->new_cat->icon);
}

void
vazaar_gui_druidcat_finish	(	GtkWidget * window, 
								VazaarDruidCat * this)
{	
	xmlNodePtr root;

	root = xmlDocGetRootElement(cats_doc);
	vazaar_gui_druidcat_resume(this);

	// If you edit a category
	if (this->editcat) {
		xmlNodePtr	cur = NULL;

		/* Check if exists another category with the same name */
		if (vazaar_cmd_strings_compare (this->editcat, this->new_cat->name) != 0) {
			node_found = NULL;
			vazaar_xml_cat_find (root->xmlChildrenNode, this->new_cat->name);
			if ( node_found) {
				vazaar_gui_show_dialog (this->dialog, GTK_STOCK_DIALOG_ERROR,
								_("There is another category with this name"), \
								  _("You must choose another."));
				return;
			}
		}
		
		// Find node to edit
    	node_found = NULL;
    	vazaar_xml_cat_find(root->xmlChildrenNode, this->editcat);
		cur = node_found;
		// Unlink node, not free
		xmlUnlinkNode(cur);
		// Set new properties
		xmlSetProp(cur, "icon", this->new_cat->icon);
		xmlSetProp(cur, "name", this->new_cat->name);
		xmlSetProp(cur, "desc", this->new_cat->desc);
		xmlSetProp(cur, "syst", this->new_cat->syst);
		// Find new_cat's parent node
		node_found = NULL;
		vazaar_xml_cat_find(root->xmlChildrenNode, this->new_cat->root);
		// Paste in the new parent node
		if (vazaar_cmd_strings_compare (this->new_cat->root, "") == 0) {
			xmlAddChildList (root, cur);
		}
		else {
			xmlAddChildList (node_found, cur);
		}
		// If changes its name, update items 
		if (vazaar_cmd_strings_compare (this->editcat, this->new_cat->name) != 0)
			vazaar_xml_item_change_category (this->editcat, this->new_cat->name);	
		vazaar_xml_cats_savefile();
	}
	else {
		/* Check if exists another category with the same name */
		node_found = NULL;
		vazaar_xml_cat_find (root->xmlChildrenNode, this->new_cat->name);
		if ( node_found) {
				vazaar_gui_show_dialog (this->dialog, GTK_STOCK_DIALOG_ERROR,
								_("There is another category with this name"), \
								  _("You must choose another."));
			return;
		}
		vazaar_xml_cat_add(this->new_cat);
	}
    vazaar_gui_catmodel_refresh(GTK_TREE_VIEW (this->treecat));
    vazaar_gui_druidcat_destroy(this);
}

void
vazaar_gui_druidCat_chkzlc_toggled(GtkWidget * nothing,
				   VazaarDruidCat * this)
{
    gboolean        parent_cat = FALSE;

    parent_cat =
	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(this->rbParentCat));
    if (parent_cat) {
	gtk_widget_set_sensitive(this->treeselcat, FALSE);
    } else {
	gtk_widget_set_sensitive(this->treeselcat, TRUE);
    }
}

gboolean
vazaar_gui_treecat_foreach_func(GtkTreeModel * model,
				GtkTreePath * path,
				GtkTreeIter * parent, gpointer user_data)
{
    gchar          *name;
    VazaarCat      *this_cat;
    GtkTreeIter     newiter;
    GdkPixbuf      *icon = NULL;

    this_cat = (VazaarCat *) user_data;
    icon = create_pixbuf(this_cat->icon);


    if (vazaar_cmd_strings_compare(this_cat->root, "") == 0) {
	gtk_tree_store_append(GTK_TREE_STORE(model), &newiter, NULL);
	gtk_tree_store_set(GTK_TREE_STORE(model), &newiter, 
			CAT_COL_IMG, icon, 
			CAT_COL_CHK, FALSE, 
			CAT_COL_NOM, this_cat->name, 
			CAT_COL_DES, this_cat->desc,
			   -1);
	return TRUE;
    }

    gtk_tree_model_get(model, parent, CAT_COL_NOM, &name, -1);

    if (vazaar_cmd_strings_compare(name, this_cat->root)) {
		gtk_tree_store_append(GTK_TREE_STORE(model), &newiter, parent);
		gtk_tree_store_set(GTK_TREE_STORE(model), &newiter, CAT_COL_IMG,
				   icon, CAT_COL_CHK, FALSE, CAT_COL_NOM,
				   this_cat->name, CAT_COL_DES, this_cat->desc,
				   -1);
		return TRUE;
    }
    g_object_unref(icon);

    return FALSE;		/* do not stop walking the store, call us with next row */
}

gint 
vazaar_cmd_cat_sort_items (VazaarItemSort *item_sort_a, VazaarItemSort *item_sort_b) {
	gint res;
	GString *a, *b;
	
	a = g_string_new (item_sort_a->item->name);
	b = g_string_new (item_sort_b->item->name);
	
	res = vazaar_cmd_strings_compare ( g_utf8_strup (a->str, a->len),  g_utf8_strup (b->str, b->len));
	//g_message ("%s - %s", g_utf8_strup (a->str, a->len),  g_utf8_strup (b->str, b->len));
	return res;
}
